#version 450 core
#extension GL_GOOGLE_include_directive : require

#define MATERIAL_DESCRIPTOR_SET 1
#define LIGHT_DESCRIPTOR_SET 3
#define SHADOW_DESCRIPTOR_SET 4
#define INDIRECT_DESCRIPTOR_SET 5

#include "math_library.glsl"
#include "material_library.glsl"
#include "light_library.glsl"

#include "per_frame.inc"

layout(location = 0) out vec4 outFragColor;

layout(location = 0) in vec3 inPos;
layout(location = 1) in vec3 inNormal;
layout(location = 2) in vec3 inTangent;
layout(location = 3) in vec2 inUV;

layout(set = 0, binding = 0) uniform Frame
{
    PerFrameData frame;
};

void main()
{
    Material material = lookup_material(inUV);
    
    if (material.opacity < EPSILON)
    {
        discard;
    }
    
    vec3 normal = lookup_normal(inUV, inNormal, inTangent);
    vec3 lighting = apply_lighting(frame.eyePosition, inPos, normal, material);

    outFragColor = vec4(lighting, 1.0);
}
